#!/usr/bin/env python
# -*- coding: utf-8 -*-
from osv import osv, fields
from datetime import datetime, timedelta
from tools.translate import _
import addons


class sisb_permits_reasons(osv.osv):

    def _isRemunerated(self, cr, uid, ids, field_names, arg, context=None):
        return {i.id: bool(i.asign_concept_id) for i in self.browse(cr, uid, ids, context=context)}

    def _check_concepts(self, cr, uid, ids, context=None):
        return all((r.asign_concept_id != r.deduct_concept_id) for r in self.browse(cr, uid, ids, context=context))

    _name = "sisb.permits.reasons"
    _description = "Reasons for permits"
    _columns = {
        'name': fields.char('Name', size=32, required=True),
        'active': fields.boolean('Active'),
        'asign_concept_id': fields.many2one('hr.concept', 'Asignation concept', domain=[('type_calc', '=', 'allocation'), ('type', '=', 'variable')]),
        'deduct_concept_id': fields.many2one('hr.concept', 'Deduction concept', domain=[('type_calc', '=', 'deduction'), ('type', '=', 'variable')]),
        'remunerated': fields.function(_isRemunerated, string="Remunerated?", type="boolean", method=True, store=False, help='Indicates if the permit is remunerated'),
        'limit': fields.integer('Limit', required=True, help="Monthly limit, in hours, allowed for each employee")
    }
    _defaults = {
        'active': lambda *a: True,
        'remunerated': lambda *a: False,
        'limit': 40
    }
    _constraints = [
        (_check_concepts, 'Concepts cannot be the same', ['Asignation concept', 'Deduction concept'])
    ]
    _sql_constraints = [('unique_name', 'unique(name)', 'A record with the same name already exists')]

    def unlink(self, cr, uid, ids, context=None):
        for r in self.browse(cr, uid, ids, context=context):
            r_ids = self.pool.get('sisb.permits').search(cr, uid, [('reason_id', '=', r.id)], context=context)
            if r_ids and not r.active:
                raise osv.except_osv(_('Error'), _('Reasons linked with permits cannot be deleted'))
            if r_ids and r.active:
                self.write(cr, uid, [r.id], {'active': False}, context=context)
            else:
                super(sisb_permits_reasons, self).unlink(cr, uid, [r.id], context=context)
        return True


sisb_permits_reasons()


class sisb_permits(osv.osv):

    def child_depts(self, cr, uid, dept_id, context=None):
        child_ids = [c.id for c in self.pool.get('hr.department').browse(cr, uid, dept_id, context=context).child_ids]
        return [dept_id] + reduce(lambda x, y: x + y, [self.child_depts(cr, uid, d, context=context) for d in child_ids], []) if child_ids else [dept_id]

    def get_states(self, cr, uid, context=None):
        return [('draft', _('Draft')), ('confirmed', _('Confirmed')), ('approved', _('Approved')), ('rejected', _('Rejected')), ('load', _('Loaded')), ('invalid', _('Invalid')), ('done', _('Done'))]

    def _check_valid_hour(self, cr, uid, ids, context=None):
        return all(i.hour_start >= 0 and i.hour_start < 24 and i.hour_end >= 0 and i.hour_end < 24 and (i.hour_start < i.hour_end if i.date_start == i.date_end and i.state != 'invalid' else True) for i in self.browse(cr, uid, ids, context=context))

    def _check_dates(self, cr, uid, ids, context=None):
        return all(i.date_start <= i.date_end for i in self.browse(cr, uid, ids, context=context))

    def _check_employee(self, cr, uid, ids, context=None):
        res = []
        for record in self.browse(cr, uid, ids, context=context):
            if record.state not in ['rejected', 'invalid']:
                cr.execute("""
                    select case when count(*) > 0 then false else true end
                    from sisb_permits
                    where employee_id = %s and id != %s and
                    date_start + (interval '1 hour' * hour_start) <= date '%s' + interval '%s hours' and
                    date_end + (interval '1 hour' * hour_end) >= date '%s' + interval '%s hours' and
                    state not in ('rejected','invalid')
                """ % (record.employee_id.id, record.id, record.date_end, record.hour_end, record.date_start, record.hour_start))
                res.append(all(i[0] for i in cr.fetchall()))
        return all(res)

    def _check_limit(self, cr, uid, ids, context=None):
        for permit in self.browse(cr, uid, ids, context=context):
            date_start = datetime.strptime(permit.date_start, '%Y-%m-%d')
            ms = '%d-%d-%d' % (date_start.year, date_start.month, 1)
            me = '%d-%d-%d' % (date_start.year + (date_start.month / 12), ((date_start.month % 12) + 1), 1)
            rids = self.search(cr, uid, [('reason_id', '=', permit.reason_id.id), ('employee_id', '=', permit.employee_id.id), ('date_start', '>=', ms), ('date_end', '<', me), ('state', 'not in', ['rejected', 'invalid'])], context=context)
            if sum([i.duration for i in self.browse(cr, uid, rids, context=context)]) > permit.reason_id.limit:
                return False
        return True

    def sites_domain(self, cr, uid, context=None):
        th = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'sisb_permits', 'group_sisb_permits_th')[1]
        pcp = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'sisb_permits', 'group_sisb_permits_pcp')[1]
        th_local = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'sisb_permits', 'group_sisb_permits_th_local')[1]
        user_groups = map(int, self.pool.get('res.users').browse(cr, uid, uid, context=context).groups_id)
        query = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'sisb_permits', 'group_sisb_permits_query')[1]
        emp_id = self.pool.get('hr.employee').search(cr, uid, [('user_id', '=', uid)], context=context)
        if not emp_id and uid != 1 and query not in user_groups and th not in user_groups and pcp not in user_groups:
            raise osv.except_osv(_('Error'), _('%s has no employee profile configured') % (self.pool.get('res.users').browse(cr, uid, uid).name))
        site_depts_ids = []  # Ids of local departments
        if th_local in user_groups and uid != 1:
            site_depts_ids = [self.pool.get('hr.employee').browse(cr, uid, emp_id, context=context)[0].hr_sede_id.id]
        return site_depts_ids

    def depts_domain(self, cr, uid, context=None):
        th = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'sisb_permits', 'group_sisb_permits_th')[1]
        query = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'sisb_permits', 'group_sisb_permits_query')[1]
        pcp = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'sisb_permits', 'group_sisb_permits_pcp')[1]
        user_groups = map(int, self.pool.get('res.users').browse(cr, uid, uid, context=context).groups_id)
        emp_id = self.pool.get('hr.employee').search(cr, uid, [('user_id', '=', uid)], context=context)
        if uid == 1 or th in user_groups or query in user_groups or pcp in user_groups:
            dept_id = 1
        elif emp_id:
            dept_id = self.pool.get('hr.employee').browse(cr, uid, emp_id, context=context)[0].department_id.id
        else:
            raise osv.except_osv(_('Error'), _('%s has no employee profile configured') % (self.pool.get('res.users').browse(cr, uid, uid).name))
        return self.child_depts(cr, uid, dept_id, context=context)

    def _getDuration(self, cr, uid, ids, field_names, arg, context=None):
        res = {}
        for record in self.browse(cr, uid, ids, context=context):
            start = datetime.strptime(record.date_start, '%Y-%m-%d') + timedelta(hours=record.hour_start)
            end = datetime.strptime(record.date_end, '%Y-%m-%d') + timedelta(hours=record.hour_end)
            res[record.id] = (end - start).total_seconds() / 3600
        return res

    _readonlyStates = {'confirmed': [('readonly', True)], 'approved': [('readonly', True)], 'rejected': [('readonly', True)], 'load': [('readonly', True)], 'invalid': [('readonly', True)], 'done': [('readonly', True)]}

    _name = 'sisb.permits'
    _rec_name = 'employee_id'
    _description = 'Employee permits'
    _order = 'id desc'
    _columns = {
        'employee_id': fields.many2one('hr.employee', 'Employee', required=True, states=_readonlyStates),
        'photo': fields.related('employee_id', 'photo', type='binary', string='Photo', readonly=True),
        'department_id': fields.related('employee_id', 'department_id', type='many2one', relation="hr.department", string='Department', readonly=True),
        'cod_number': fields.related('employee_id', 'cod_number', type='char', string='Cod number', readonly=True),
        'date_start': fields.date('Date start', required=True, states=_readonlyStates),
        'date_end': fields.date('Date end', required=True, states=_readonlyStates),
        'hour_start': fields.float('Hour start', required=True, states=_readonlyStates, help="El formato de hora puede ser un numero real entre 0 y 23.99, que se representan como hora militar, o tambien se puede usar de manera convencional con el separador de dos puntos, entre las 0:00 y las 23:59"),
        'hour_end': fields.float('Hour end', required=True, states=_readonlyStates, help="El formato de hora puede ser un numero real entre 0 y 23.99, que se representan como hora militar, o tambien se puede usar de manera convencional con el separador de dos puntos, entre las 0:00 y las 23:59"),
        'reason_id': fields.many2one('sisb.permits.reasons', 'Reason', required=True, states=_readonlyStates),
        'state': fields.selection(get_states, 'State', readonly=True, required=True),
        'remunerated': fields.related('reason_id', 'remunerated', type='boolean', string="Remunerated?", readonly=True),
        'duration': fields.function(_getDuration, string="Duration", type="float", method=True, store=False, help='Calculated duration of the permit, in hours'),
        'requester': fields.many2one('res.users', 'Requester', readonly=True, help="User who requested the permit"),
        'confirmed_by': fields.many2one('res.users', 'Confirmed by', readonly=True, help="User who confirmed this permit"),
        'approved_by': fields.many2one('res.users', 'Approved by', readonly=True, help="User who approved this permit"),
        'rejected_by': fields.many2one('res.users', 'Rejected by', readonly=True, help="User who rejected this permit"),
        'variable_concept_id': fields.many2one('hr.variable.concepts', 'Variable Concept')
    }
    _defaults = {
        'state': lambda *a: 'draft',
        'requester': lambda self, cr, uid, context=None: uid,
        'photo': open(addons.get_module_resource('hr', 'images', 'photo.png'), 'rb').read().encode('base64')
    }
    _constraints = [
        (_check_valid_hour, 'Hour not valid', ['Hour start', 'Hour end']),
        (_check_dates, 'Dates not valid', ['Date start', 'Date end']),
        (_check_employee, 'Employee already has a permit for the selected period', ['Employee']),
        (_check_limit, 'Employee has reached the monthly limit allowed for this kind of permit', ['Duration']),
        (lambda self, cr, uid, ids, context=None: all(i.duration <= 24 for i in self.browse(cr, uid, ids, context=context)), 'Duration cannot be greater than 24 hours', ['Dates fields'])
    ]

    def backdraft(self, cr, uid, ids, context=None):
        for record in self.browse(cr, uid, ids, context=context):
            if record.state == 'confirmed':
                context = context or {}
                context.update({'backdraft': True})
                self.write(cr, uid, [record.id], {'state': 'draft'}, context=context)
        return True

    def confirm(self, cr, uid, ids, context=None):
        for record in self.browse(cr, uid, ids, context=context):
            if record.state == 'draft':
                self.write(cr, uid, [record.id], {'state': 'confirmed', 'confirmed_by': uid}, context=context)
        return True

    def approve(self, cr, uid, ids, context=None):
        for record in self.browse(cr, uid, ids, context=context):
            if record.state == 'confirmed':
                self.write(cr, uid, [record.id], {'state': 'approved', 'approved_by': uid}, context=context)
        return True

    def reject(self, cr, uid, ids, context=None):
        for record in self.browse(cr, uid, ids, context=context):
            if record.state == 'confirmed':
                self.write(cr, uid, [record.id], {'state': 'rejected', 'rejected_by': uid}, context=context)
        return True

    def employee_onchange(self, cr, uid, ids, emp_id, context):
        if emp_id:
            emp_brw = self.pool.get('hr.employee').browse(cr, uid, emp_id, context=context)
            cod_number, dept_id, photo = emp_brw.cod_number, emp_brw.department_id.id, emp_brw.photo
        else:
            cod_number = dept_id = None
        return {
            'value': {
                'department_id': dept_id,
                'cod_number': cod_number,
                'photo': photo
            }
        }

    def reason_onchange(self, cr, uid, ids, reason_id, context):
        return {
            'value': {
                'remunerated': self.pool.get('sisb.permits.reasons').browse(cr, uid, reason_id, context=context).remunerated if reason_id else None
            }
        }

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        args = [(f, 'in', self.child_depts(cr, uid, v)) if f == 'department_id' and o == '=' else (f, o, v) for f, o, v in args]
        site_domain = self.sites_domain(cr, uid, context=context)
        emp_domain = ('employee_id.hr_sede_id', 'in', site_domain) if site_domain else ('department_id', 'in', self.depts_domain(cr, uid, context=context)) if uid != 1 else ('employee_id', '!=', 0)
        args.append(emp_domain)
        return super(sisb_permits, self).search(cr, uid, args, offset=offset, limit=limit, order=order, context=context, count=count)

    def copy(*a, **k):
        raise osv.except_osv(_('Error'), _('Duplication is not allowed'))

    def unlink(self, cr, uid, ids, context=None):
        for i in self.browse(cr, uid, ids, context=context):
            if i.state != 'draft':
                raise osv.except_osv(_('Error'), _('Permits can only be deleted on draft state'))
        return super(sisb_permits, self).unlink(cr, uid, ids, context=context)

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
        view = super(sisb_permits, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar, submenu=submenu)
        if view_type in ['search', 'form']:
            site_domain = self.sites_domain(cr, uid, context=context)
            if site_domain:
                emp_domain = ('hr_sede_id', 'in', site_domain)
                dept_domain = ('member_ids.hr_sede_id', 'in', site_domain)
            else:
                emp_domain = ('department_id', 'in', self.depts_domain(cr, uid, context=context))
                dept_domain = ('id', 'in', self.depts_domain(cr, uid, context=context))
            view['fields']['employee_id']['domain'] = [emp_domain, ('state', 'not in', ['egresado', 'borrador'])]
            view['fields']['department_id']['domain'] = [dept_domain]
        return view


sisb_permits()
