# -*- coding: utf-8 -*-
"""
Módulo utilizado para el registro de la recepción de cada buque y 
el registro del destino final del producto proveniente del buque.
"""

import time
from datetime import datetime, timedelta
import netsvc
from osv import fields, osv
from tools import config
from tools.translate import _
import decimal_precision as dp

#----------------------------------------------------------
#  Datos de la recepción del Buque
#----------------------------------------------------------


class sisb_ship_reception(osv.osv):
    _name = "sisb.ship.reception"
    _description = "Buque"
    _order = "arrival_date desc"
    
    
    
    #Para saber la disponibilidad de lo descargado
    def _product_available(self, cr, uid, ids, name, arg, context=None):
        if context is None:
            context = {}
            
        res = {}
        
        for ship_reception in self.browse(cr, uid, ids, context=context):
            s_id = ship_reception.id
            res[s_id] = ship_reception.product_qty
            for mov_det in ship_reception.ship_reception_lines:
                #Se toma en cuenta si son ubicaciones internas
                #~ if not mov_det.transaction_type_id.transfer_between_warehousing:
                res[s_id] -= mov_det.product_qty
        return res
        
    def _get_picking_internal(self, cr, uid, ids, name, arg, context=None):
        if context is None:
            context = {}

        res = {}
        for ship_reception in self.browse(cr, uid, ids, context=context):
            s_id = ship_reception.id
            res[s_id] = []
            if not ship_reception.ship_reception_lines:
                continue
            data_lines = [x.picking_out_ids for x in ship_reception.ship_reception_lines]

            l=[]
            for pick in data_lines:
                for pl in pick:
                    l.append(pl.id)
                    res[s_id] = l
        return res
        
       
    _columns = {
        'ship_id': fields.many2one('sisb.ship.master', 'Ship Master', required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'name': fields.char('Name', size=64, select=True, required=True),
        'reference': fields.char('Ship Reference', size=64, readonly=True, states={'draft': [('readonly', False)]}, help="The reference of this ship."),
        'state': fields.selection([('draft', 'Draft'), ('waiting', 'Waiting'), ('confirmed', 'Discharged'), ('assigned', 'Available'), ('done', 'Done'), ('cancel', 'Cancelled')], 'State', readonly=True, select=True,
              help='When the ship reception is created it is in the \'Draft\' state.\n After that, it is set to \'Discharged\' state if the user set the quantity the product.\n When products are assigned the partner it is set to \'Available\'.\n When the ship is totally sent the state is \'Done\'.\
              \nThe state is \'Waiting\' if the ship reception is waiting for arrival.'),
        'product_id': fields.many2one('sisb.product.master', 'Product Master', required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'product_uom_qty': fields.float('Expected Quantity', digits_compute=dp.get_precision('Account'), required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'product_uom': fields.many2one('product.uom', 'Unit of Measure', required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'arrival_date': fields.date('Arrival date', help="Arrival date", select=True, required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'real_arrival_date': fields.date('Real Arrival date', help="Real Arrival date", select=True, states={'draft': [('readonly', True)], 'waiting': [('readonly', True)], 'confirmed': [('readonly', True)], 'assigned': [('readonly', True)], 'done': [('readonly', True)], 'cancel': [('readonly', True)]}),
        'location_dest_id': fields.many2one('stock.location', 'Destination Location', readonly=True, select=True, help="Location where the system will stock the product."),
        'product_qty': fields.float('Real Quantity', digits_compute=dp.get_precision('Account'), states={'draft': [('readonly', True)], 'waiting': [('readonly', True)], 'confirmed': [('readonly', True)], 'assigned': [('readonly', True)], 'done': [('readonly', True)], 'cancel': [('readonly', True)]}),
        'product_available': fields.function(_product_available, method=True, type='float', string='Product Available', digits_compute=dp.get_precision('Account'), store = False), 
        'cost_unit': fields.float('Unit Cost', digits_compute= dp.get_precision('Account'), readonly=True, help="Technical field used to record the product cost set by the user during a picking confirmation (when average price costing method is used)"),
        'is_cost': fields.boolean('Is Cost', select=True, readonly=True),
        'picking_ids': fields.one2many('stock.picking', 'sisb_ship_id', 'Stock Picking Related', readonly=True, help="This is the list of picking that have been generated from the ship reception document."),
        'ship_reception_lines': fields.one2many('sisb.ship.reception.detail', 'ship_reception_id', 'Detail Moves', states={'done': [('readonly', True)], 'cancel': [('readonly', True)]}),
        'note': fields.text('Notes', required=True, states={'confirmed': [('readonly', True)], 'assigned': [('readonly', True)], 'done': [('readonly', True)], 'cancel': [('readonly', True)]}),
        'note_discharged': fields.text('Discharged Notes', readonly=True),
        'company_id': fields.many2one('res.company', 'Company', required=True),
        'picking_internal_ids':fields.function(_get_picking_internal, method=True, type='many2many', relation='stock.picking', string='Stock Picking Internal'),
        'log_ship_reception_lines': fields.one2many('sisb.log.ship.reception', 'ship_reception_id', 'Log ship reception', readonly=True),
    }
    
    _defaults = {
        'reference': lambda self, cr, uid, context: 'BQ/',
        'state': 'draft',
        'cost_unit': 0.00,
        'is_cost': False,
        'company_id': lambda self,cr,uid,c: self.pool.get('res.company')._company_default_get(cr, uid, 'sisb.ship.reception', context=c),
    }
    
    _sql_constraints = [
        ('uniq_ship_id_arrival_date_relation', 'unique (ship_id, arrival_date)', 'The ship and the arrival date must be unique !')
    ]
    
    #para saber si el albaran de entrada ha pasado al estado realizado
    def is_pick_processed(self, cr, uid, ship_id, context=None):
        if context is None:
            context = {}
        ship_id=ship_id[0]
        ship_pick_in_brw = self.browse(cr, uid, ship_id, context=context).picking_ids
        #Garantizar que se tome el primer albaran generado
        ship_pick_in_brw = sorted(ship_pick_in_brw, key=lambda ship_pick: ship_pick.id)
        if ship_pick_in_brw[0].type == 'in' and ship_pick_in_brw[0].state == 'done':
            return True
                
        return False
    
    def onchange_ship_id(self, cr, uid, ids, ship_id, arrival_date):
        result = {}
        if ship_id:
            valor02 = self.pool.get('sisb.ship.master').browse(cr, uid, ship_id).name
            
        else:
            valor02 = ''
        
        result = {'value': {
                'name': valor02,
                }
            }
            
        return result
        
        
    def onchange_product_id(self, cr, uid, ids, product_id):
        result = {}
        if product_id:
            brw_product = self.pool.get('sisb.product.master').browse(cr, uid, product_id)
            uom_product_id = brw_product.product_id and brw_product.product_id.uom_id.id  or ''
            result = {'value': {
                'product_uom': uom_product_id,
                }
            }
        else:
            result = {'value': {
                'product_uom': False,
                }
            }
        return result
        
    #Condiciones para pasar a action_cancel
    def valida_cancel(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        return True
        
    def action_cancel(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        if self.valida_cancel(cr, uid, ids, context=context):
            self.write(cr, uid, ids, {'state':'cancel'}, context=context)
        return True
        
    #Condiciones para pasar a waitting
    def valida_waitting(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        for ship_reception in self.browse(cr, uid, ids, context=context):
            today = datetime.now()
            now_day = datetime(today.year, today.month, today.day,0,0,0)
            someday = datetime.strptime(ship_reception.arrival_date, '%Y-%m-%d')
            diff = someday - now_day
            
            if diff.days < 0:
                raise osv.except_osv(_('Invalid action !'), _('Date invalidate.'))
                
            if ship_reception.product_uom_qty <= 0:
                raise osv.except_osv(_('Invalid action !'), _('Quantity invalidate.'))
        
        return True
        
    def action_waitting(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        if self.valida_waitting(cr, uid, ids, context=context):
            for ship_reception in self.browse(cr, uid, ids, context=context):
                a_date = datetime.strptime(ship_reception.arrival_date, '%Y-%m-%d')
                ref = 'BQ/' + ship_reception.name + '_' + str(a_date.day).zfill(2) + '_' + str(a_date.month).zfill(2) + '_' + str(a_date.year)
                self.write(cr, uid, ship_reception.id, {'state':'waiting', 'reference':ref}, context=context)
                mess_date = str(a_date.day).zfill(2) + '-' + str(a_date.month).zfill(2) + '-' + str(a_date.year)
                #Notificaciones en el log
                message = 'Se confirma en el sistema la llegada del buque %s a la fecha %s con %s %s del producto %s ' % (ship_reception.name, mess_date, ship_reception.product_uom_qty, ship_reception.product_uom.name, ship_reception.product_id.name)
                registro =  {
                    'name': ref + '/Esperando',
                    'ship_reception_id': ship_reception.id,
                    'transition': 'waiting',
                    'description': message,
                    'responsible_uid': uid,
                    'is_notified': False,
                }
                
                resultado = self.pool.get('sisb.log.ship.reception').create(cr, uid, registro, context=context)
        return True
        
    #Condiciones para pasar a confirmed
    def valida_confirmed(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        return True
        
    def action_confirmed(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        if self.valida_confirmed(cr, uid, ids, context=context):
            
            for ship_reception in self.browse(cr, uid, ids, context=context):
                self.write(cr, uid, ship_reception.id, {'state':'confirmed'}, context=context)
                a_date = datetime.strptime(ship_reception.real_arrival_date, '%Y-%m-%d')
                mess_date = str(a_date.day).zfill(2) + '-' + str(a_date.month).zfill(2) + '-' + str(a_date.year)
                #Notificaciones en el log
                message = 'Se actualizaron los datos de descarga en el sistema del buque %s a la fecha %s con %s %s del producto %s ' % (ship_reception.name, mess_date, ship_reception.product_qty, ship_reception.product_uom.name, ship_reception.product_id.name)
                registro =  {
                    'name': ship_reception.reference + '/Descargado',
                    'ship_reception_id': ship_reception.id,
                    'transition': 'confirmed',
                    'description': message,
                    'responsible_uid': uid,
                    'is_notified': False,
                }
                
                resultado = self.pool.get('sisb.log.ship.reception').create(cr, uid, registro, context=context)

        return True
        
    #Condiciones para pasar a assigned
    def valida_assigned(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        return True
        
    def action_assigned(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        if self.valida_assigned(cr, uid, ids, context=context):
            self.write(cr, uid, ids, {'state':'assigned'}, context=context)
            #Se verifica la existencia para cambiarlo al estado Realizado (done)
            for ship_reception in self.browse(cr, uid, ids, context=context):
                if ship_reception.product_available == 0.00:
                    resultado = self.pool.get('sisb.ship.reception').action_done(cr,uid,[ship_reception.id],context=context)
                    
        return True
        
    #Condiciones para pasar a done
    def valida_done(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        return True
        
    def action_done(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        if self.valida_done(cr, uid, ids, context=context):
            self.write(cr, uid, ids, {'state':'done'}, context=context)
        return True
    
    def action_cost_assigned(self, cr, uid, ship_id, cost_unit, context=None):
        
        if context is None:
            context = {}
            
        reception_ship_detail_obj = self.pool.get('sisb.ship.reception.detail')
        data_update = {
            'cost_unit':  cost_unit,
            'is_cost': True,
        }
        resultado01 = False
        #CREACION DEL ALABARAN DE ENTRADA
        if self.action_picking_create(cr, uid, [ship_id], cost_unit, 'in_invoice', context=context):
            #CREACION DE LOS ALBARANES INTERNOS/SALIDA
            #Para procesar alabaranes internos debe haber una condicion de que el albaran de entrada se ha procesado
            if self.is_pick_processed:
                reception_ship_detail_ids = reception_ship_detail_obj.search(cr, uid, [('ship_reception_id', '=', ship_id)], context=context)
                
                if reception_ship_detail_obj.action_picking_create_from_detail(cr, uid, reception_ship_detail_ids, context=context):
                    resultado_end = self.write(cr,uid,ship_id,data_update,context=context)
                    
                    if resultado_end:
                        ship_reception = self.browse(cr, uid, ship_id, context=context)
                        a_date = datetime.now()
                        mess_date = str(a_date.day).zfill(2) + '-' + str(a_date.month).zfill(2) + '-' + str(a_date.year)
                        #Notificaciones en el log
                        message = 'Se actualizaron los datos para asignar costo %s en el sistema al buque %s a la fecha %s con %s %s del producto %s ' % (cost_unit, ship_reception.name, mess_date, ship_reception.product_qty, ship_reception.product_uom.name, ship_reception.product_id.name)
                        #Programado de este modo por porblemas de traduccion
                        type_list = {
                            'confirmed': "Descargado",
                            'assigned': "Disponible",
                            'done': "Realizado",
                        }
                        
                        registro =  {
                            'name': ship_reception.reference + "/" + type_list.get(ship_reception.state, "Documento: " + ship_reception.state)  + " /costo",
                            'ship_reception_id': ship_reception.id,
                            'transition': ship_reception.state,
                            'description': message,
                            'responsible_uid': uid,
                            'is_notified': False,
                        }
                        
                        resultado = self.pool.get('sisb.log.ship.reception').create(cr, uid, registro, context=context)

                        resultado01 = True
                        
        return resultado01
    
    
    def action_picking_create(self, cr, uid, ids, cost_unit, tipo='out_refund', context=None):
        wf_service = netsvc.LocalService("workflow")
        pick_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get('stock.move')
        product_obj = self.pool.get('product.product')
        company = self.pool.get('res.users').browse(cr, uid, uid).company_id
        for rec_ship in self.browse(cr, uid, ids, context={}):
            if not rec_ship.picking_ids:
                output_id = rec_ship.location_dest_id and rec_ship.location_dest_id.id or False
                if not output_id:
                    raise osv.except_osv(_('Invalid action !'), _('No location configured in the reception ship or do not have permission to access the location.'))
                
                proc_id = False
                date_planned = datetime.now()
                date_planned = (date_planned - timedelta(days=company.security_lead)).strftime('%Y-%m-%d %H:%M:%S')

                if rec_ship.product_id.product_id and rec_ship.product_id.product_id.product_tmpl_id.type in ('product', 'consu'):
                    if tipo == 'out_refund':
                        location_id = rec_ship.company_id.partner_id.property_stock_customer.id
                    else:
                        location_id = rec_ship.company_id.partner_id.property_stock_supplier.id
                        
                    pick_name = self.pool.get('ir.sequence').get(cr, uid, 'stock.picking.in')
                    
                    addr = self.pool.get('res.partner').address_get(cr, uid, [rec_ship.company_id.partner_id.id], ['invoice'])
                    
                    picking_id = pick_obj.create(cr, uid, {
                        'name': pick_name,
                        'origin': rec_ship.reference,
                        'type': 'in',
                        'state': 'auto',
                        'move_type': 'one',
                        'sisb_ship_id': rec_ship.id,
                        'address_id': addr['invoice'] or False,
                        'invoice_state': 'none',
                        'company_id': rec_ship.company_id.id,
                        'note': rec_ship.note,
                    })
                    
                    move_id = move_obj.create(cr, uid, {
                        'name': product_obj.name_get(cr, uid, [rec_ship.product_id.product_id.id])[0][1][:64],
                        'picking_id': picking_id,
                        'product_id': rec_ship.product_id.product_id.id,
                        'date': date_planned,
                        'date_expected': date_planned,
                        'product_qty': rec_ship.product_qty,
                        'product_uom': rec_ship.product_uom.id,
                        'product_uos_qty': rec_ship.product_qty,
                        'product_uos': rec_ship.product_uom.id,
                        'address_id': addr['invoice'] or False,
                        'location_id': location_id,
                        'location_dest_id': output_id,
                        'tracking_id': False,
                        'state': 'draft',
                        'company_id': rec_ship.company_id.id,
                    })
                
                if picking_id:
                    wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
                    pick_obj.force_assign(cr, uid, [picking_id], context)
                    
                    #procesar el albaran de entrada asignando nuevo costo
                    partial_datas = {
                        'delivery_date': date_planned,
                    }
                    partial_datas['move%s' % (move_id)] = {
                            'prodlot_id': False,
                            'product_id': rec_ship.product_id.product_id.id,
                            'product_uom': rec_ship.product_uom.id,
                            'product_qty': rec_ship.product_qty,
                            'product_currency': rec_ship.company_id.currency_id.id,
                            'product_price': cost_unit
                        }
                        
                    pick_obj.do_partial(cr, uid, [picking_id], partial_datas, context=context)
                    
        return True
        
    def copy(self, cr, uid, ids, defaults, context=None):
        raise osv.except_osv(_('Error !'), _('You can not duplicated'))
        
    def unlink(self, cr, uid, ids, context=None):
        raise osv.except_osv(_('Error !'), _('You can not deleted'))
    
sisb_ship_reception()



#----------------------------------------------------------
#  Datos del envío del destino final del producto
#----------------------------------------------------------

class sisb_ship_reception_detail(osv.osv):
    _name = "sisb.ship.reception.detail"
    _order = "date desc"
    
    def _check_permissions_to_invoice(self, cr, uid, ids, field_name, arg, context):
        res = {}
        
        for rec_ship_detail in self.browse(cr, uid, ids, context=context):
            perm_mark = False
            perm_invoice = False
            albaran_done = False
            #Se activa solo si el albaran asociado esta creado en 'done' y el control factura esta en 'none'.
            if rec_ship_detail.state == 'done':
                picking_out_brw = rec_ship_detail.picking_out_ids
                if picking_out_brw:
                    #Garantizar que se tome el primer albaran generado
                    picking_out_brw = sorted(picking_out_brw, key=lambda picking_out: picking_out.id)
                    pick_state = picking_out_brw[0].state
                    pick_invoice_state = picking_out_brw[0].invoice_state
                    if pick_state == 'done' and pick_invoice_state == 'none':
                        albaran_done = True
            if albaran_done:
                #Si el registro ya esta marcado para facturar y
                #Si el tipo de transaccion es para facturar y la factura es manual, se habilita para marcar
                if rec_ship_detail.invoice_state != '2binvoiced' and rec_ship_detail.transaction_type_id.is_invoiced and rec_ship_detail.transaction_type_id.type_invoice_transaction == 'mark_to_invoice':
                    perm_mark = True
                    
                elif rec_ship_detail.invoice_state == '2binvoiced':
                    perm_invoice = True
            
            res[rec_ship_detail.id] = { 'permissions_mark_invoiced': perm_mark,
                                        'permissions_invoiced': perm_invoice,
                                    }
        return res
        
    
    _columns = {
        'name': fields.related('ship_reception_id', 'reference', type='char', size=64, string='Name', store=True, select=True),
        'partner_id': fields.many2one('res.partner', 'Customer', required=True, select=True),
        'partner_invoice_id': fields.many2one('res.partner.address', 'Invoice Address', required=True, help="Invoice address for current partner."),
        'product_id': fields.related('ship_reception_id','product_id',type='many2one',relation='sisb.product.master',string='Product', store=True, select=True),
        'product_qty': fields.float('Quantity', digits_compute=dp.get_precision('Account'), required=True),
        'product_uom': fields.related('ship_reception_id','product_uom',type='many2one',relation='product.uom',string='Product UoM', store=True, select=True),
        'ship_reception_id': fields.many2one('sisb.ship.reception', 'Ship Reception', select=True, required=True),
        'transaction_type_id': fields.many2one('sisb.transaction.type', 'Transaction Type', select=True, required=True),
        'location_id': fields.many2one('stock.location', 'Source Location', required=True, select=True, help="Sets a location if you produce at a fixed location. This can be a partner location if you subcontract the manufacturing operations."),
        'location_dest_id': fields.many2one('stock.location', 'Destination Location', required=True, select=True, help="Location where the system will stock the finished products."),
        'date': fields.date('Date', help="Date", select=True, required=True),
        'note': fields.text('Notes', required=True),
        'picking_out_ids': fields.one2many('stock.picking', 'sisb_detail_ship_id', 'Stock Picking Related', readonly=True, help="This is the list of picking that have been generated from the detail ship reception document."),
        'printers': fields.boolean('print'),
        'state': fields.selection([('draft', 'Draft'), ('done', 'Done'), ('cancel', 'Cancelled')], 'State', readonly=True, select=True,
              help='When the detail assignment is created it is in the \'Draft\' state.\n After that.\n When the product are assigned the cost and picking is done it is set to \'Done\'.'),
        'invoice_state': fields.selection([
            ("invoiced", "Invoiced"),
            ("2binvoiced", "To be Invoiced"),
            ("none", "Not Applicable")], "Invoice control",
            select=True, readonly=True),
        'permissions_mark_invoiced': fields.function(_check_permissions_to_invoice, method=True, string='Permissions to mark invoice', type='boolean', multi = 'True'),
        'permissions_invoiced': fields.function(_check_permissions_to_invoice, method=True, string='Permissions to invoice', type='boolean', multi = 'True'),
        'invoice_ids': fields.many2many('account.invoice', 'sisb_ship_reception_detail_invoice_rel', 'sisb_ship_reception_detail_id', 'invoice_id', 'Invoices'),
        'valid_item':fields.integer('Valid Item'),
    }
    
    _defaults = {
        'partner_invoice_id': lambda self, cr, uid, context: context.get('partner_id', False) and self.pool.get('res.partner').address_get(cr, uid, [context['partner_id']], ['invoice'])['invoice'],
        'printers':False,
        'state': 'draft',
        'invoice_state': 'none',
    }
    
    _sql_constraints = [
        ('uniq_ship_recept_detail_id_valid_item_relation', 'unique (ship_reception_id, valid_item)', 'The ship reception detail and the valid item must be unique !')
    ]
    
    def printer(self, cr, uid, ids, context=None):
        
        if context is None: context = {}
        
        return {
            'name':_("Moviento de buque"),
            'view_mode': 'form',
            'view_id': False,
            'view_type': 'form',
            'res_model': 'wizard.reception.ship',
            'res_id': ids[0],
            'type': 'ir.actions.act_window',
            'nodestroy': True,
            'target': 'new',
            'domain': '[]',
            'context': dict(context, active_ids=ids)
            }
            
    def action_check_invoice_from_detail(self, cr, uid, ids, context=None):
        
        resultado_end_detail = self.write(cr,uid,ids,{'invoice_state':'2binvoiced'},context=context)
        
        return True
    
    def get_product_category_account(self,cr, uid, categ_id, fieldname,context=None):
        if context is None:
            context = {}
        categ_brw=self.pool.get('product.category').browse(cr, uid, categ_id, context=context)
        account_id = getattr(categ_brw,fieldname) and getattr(categ_brw,fieldname).id or False
        if not account_id and categ_brw.parent_id:
            account_id = self.get_product_category_account(cr, uid, categ_brw.parent_id.id, fieldname,context=context)
        return account_id
    
    def action_create_invoice_from_detail(self, cr, uid, ids, journal_id=False, invoice_type='out_invoice', context=None):
        #Crear factura
        """ Crea la factura de los detalles de movimientos para facturar
        @param ids: ids de detalle de recepcion
        @param journal_id: diario de venta para la factura
        @param invoice_type: Tipo de factura que debe crearse
        @return: Ids de la factura creada para el detalle de movimiento
        """

        if context is None:
            context = {}
        
        journal_obj = self.pool.get('account.journal')
        invoice_obj = self.pool.get('account.invoice')
        invoice_line_obj = self.pool.get('account.invoice.line')
        picking_obj = self.pool.get('stock.picking')
        product_obj = self.pool.get('product.product')

        res = {}
        for detalle in self.browse(cr, uid, ids, context=context):
            if detalle.invoice_state == '2binvoiced':
                journal_brw = journal_obj.browse(cr, uid, journal_id, context=context)
                payment_term_id = journal_brw.sale_shop_id.payment_default_id and journal_brw.sale_shop_id.payment_default_id.id or False
                
                account_id = detalle.partner_id.property_account_receivable.id
                address_contact_id = detalle.partner_invoice_id.id
                address_invoice_id = detalle.partner_invoice_id.id
                address_shipping_id = detalle.partner_invoice_id.id
                
                comment = detalle.note
                origin = detalle.name
                
                invoice_vals = {
                    'name': detalle.name,
                    'origin': origin or '',
                    'type': invoice_type,
                    'account_id': account_id,
                    'partner_id': detalle.partner_id.id,
                    'address_invoice_id': address_invoice_id,
                    'address_contact_id': address_contact_id,
                    'address_shipping_id': address_shipping_id,
                    'comment': comment,
                    'payment_term': payment_term_id,
                    'fiscal_position': detalle.partner_id.property_account_position.id,
                    'date_invoice': context.get('date_inv',False),
                    'company_id': detalle.ship_reception_id.company_id.id,
                    'user_id': uid
                }
                
                if journal_id:
                    invoice_vals['journal_id'] = journal_id
                invoice_id = invoice_obj.create(cr, uid, invoice_vals, context=context)
                res[detalle.id] = invoice_id
                
                account_id = detalle.product_id.product_id.product_tmpl_id.property_account_income.id
                if not account_id:
                    account_id = detalle.product_id.product_id.categ_id.property_account_income_categ.id
                if not account_id:
                    account_id = self.get_product_category_account(cr,uid,detalle.product_id.product_id.categ_id.id,"property_account_income_categ",context=context)
                if not account_id:
                    raise osv.except_osv(_('Error !'),
                            _('There is no income account defined ' \
                                    'for this product: "%s" (id:%d)') % \
                                    (detalle.product_id.product_id.name, detalle.product.product_id.id,))
                
                account_analytic_id = journal_brw.sale_shop_id.project_id and journal_brw.sale_shop_id.project_id.id or False
                
                taxes_ids = [x.id for x in detalle.product_id.product_id.taxes_id]
                
                product_name = product_obj.name_get(cr, uid, [detalle.product_id.product_id.id])
                product_name = product_name and product_name[0][1]
                invoice_line_id = invoice_line_obj.create(cr, uid, {
                    'name': product_name + ' (' + detalle.name + ')',
                    'origin': origin or '',
                    'invoice_id': invoice_id,
                    'uos_id': detalle.product_id.product_id.uom_id.id,
                    'product_id': detalle.product_id.product_id.id,
                    'account_id': account_id,
                    'price_unit': detalle.product_id.product_id.list_price,
                    'discount': 0.00,
                    'quantity': detalle.product_qty,
                    'invoice_line_tax_id': [(6, 0,taxes_ids)],
                    'account_analytic_id': account_analytic_id,
                    'concept_id': False,
                    'note': detalle.note,
                }, context=context)
                
                invoice_obj.button_compute(cr, uid, [invoice_id], context=context)

                self.write(cr, uid, [detalle.id], {
                    'invoice_ids': [(4, invoice_id)],
                    'invoice_state': 'invoiced',
                    }, context=context)
            
        
                #*** cambiar estado del albaran
                picking_out_brw = detalle.picking_out_ids
                if picking_out_brw:
                    #Garantizar que se tome el primer albaran generado
                    picking_out_brw = sorted(picking_out_brw, key=lambda picking_out: picking_out.id)
                    picking_obj.write(cr, uid, picking_out_brw[0].id, {'invoice_state': 'invoiced'}, context=context)
                else:
                    raise osv.except_osv(_('Error !'), _('There is no stock picking associated.'))
                    
                #Notificaciones en el log
                message = 'Se ha creado la factura del buque %s a la empresa %s con %s %s del producto %s. Id: %s ' % (detalle.ship_reception_id.reference, detalle.partner_id.name, detalle.product_qty, detalle.product_uom.name, detalle.product_id.name, detalle.id)
                registro =  {
                    'name': detalle.ship_reception_id.reference + ' /factura id:' + str(detalle.id),
                    'ship_reception_id': detalle.ship_reception_id.id,
                    'transition': 'invoiced',
                    'description': message,
                    'responsible_uid': uid,
                    'is_notified': False,
                }
                
                resultado = self.pool.get('sisb.log.ship.reception').create(cr, uid, registro, context=context)
            
        return res
        
        
    def action_picking_create_from_detail(self, cr, uid, ids, context=None):
        wf_service = netsvc.LocalService("workflow")
        pick_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get('stock.move')
        company = self.pool.get('res.users').browse(cr, uid, uid).company_id
        for rec_ship_detail in self.browse(cr, uid, ids, context={}):
            if not rec_ship_detail.picking_out_ids:
                output_id = rec_ship_detail.location_dest_id and rec_ship_detail.location_dest_id.id or False
                if not output_id:
                    raise osv.except_osv(_('Invalid action !'), _('No location configured in the detail reception ship or do not have permission to access the location.'))
                
                proc_id = False
                date_planned = datetime.now()
                date_planned = (date_planned - timedelta(days=company.security_lead)).strftime('%Y-%m-%d %H:%M:%S')
                
                #Determinar si es un albaran interno o de salida
                if rec_ship_detail.transaction_type_id.is_own:
                    tipo='internal'
                else:
                    tipo='out'
                
                if rec_ship_detail.product_id.product_id and rec_ship_detail.ship_reception_id.product_id.product_id.product_tmpl_id.type in ('product', 'consu'):
                    
                    location_id = rec_ship_detail.location_id.id
                    
                    if tipo == 'out':
                        pick_name = self.pool.get('ir.sequence').get(cr, uid, 'stock.picking.out')
                    else:
                        pick_name = self.pool.get('ir.sequence').get(cr, uid, 'stock.picking.internal')
                    
                    picking_id = self.pool.get('stock.picking').create(cr, uid, {
                        'name': pick_name,
                        'origin': rec_ship_detail.name,
                        'type': tipo,
                        'state': 'auto',
                        'move_type': 'one',
                        'sisb_detail_ship_id': rec_ship_detail.id,
                        'address_id': rec_ship_detail.partner_invoice_id and rec_ship_detail.partner_invoice_id.id or False,
                        'invoice_state': 'none',
                        'company_id': rec_ship_detail.ship_reception_id.company_id.id,
                        'note': rec_ship_detail.note,
                    })
                    
                    move_id = self.pool.get('stock.move').create(cr, uid, {
                        'name': self.pool.get('product.product').name_get(cr, uid, [rec_ship_detail.product_id.product_id.id])[0][1][:64],
                        'picking_id': picking_id,
                        'product_id': rec_ship_detail.product_id.product_id.id,
                        'date': date_planned,
                        'date_expected': date_planned,
                        'product_qty': rec_ship_detail.product_qty,
                        'product_uom': rec_ship_detail.product_uom.id,
                        'product_uos_qty': rec_ship_detail.product_qty,
                        'product_uos': rec_ship_detail.product_uom.id,
                        'address_id': rec_ship_detail.partner_invoice_id and rec_ship_detail.partner_invoice_id.id or False,
                        'location_id': location_id,
                        'location_dest_id': output_id,
                        'tracking_id': False,
                        'state': 'draft',
                        'company_id': rec_ship_detail.ship_reception_id.company_id.id,
                    })
                
                if picking_id:
                    wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
                    
                    #Procesar automatico
                    
                    reservado = pick_obj.action_assign(cr, uid, [picking_id])
                    verificar_reserva = pick_obj.test_assigned(cr, uid, [picking_id])
                    if reservado and verificar_reserva: #Si hay disponibilidad y pasa a reservado
                        partial_datas = {}
                    
                        partial_datas['move%s' % (move_id)] = {
                                'prodlot_id': False,
                                'product_id': rec_ship_detail.product_id.product_id.id,
                                'product_uom': rec_ship_detail.product_uom.id,
                                'product_qty': rec_ship_detail.product_qty,
                            }
                            
                        pick_obj.do_partial(cr, uid, [picking_id], partial_datas, context=context)
                    else:
                        raise osv.except_osv(_('Invalid action !'), _('Proceso incompleto. El albaran interno no se completo por existencias incompletas.'))
                
                resultado_end_detail = self.write(cr,uid,rec_ship_detail.id,{'state':'done'},context=context)
            
        return True
        
    def copy(self, cr, uid, ids, defaults, context=None):
        raise osv.except_osv(_('Error !'), _('You can not duplicated'))
        
    def unlink(self, cr, uid, ids, context=None):
        raise osv.except_osv(_('Error !'), _('You can not deleted'))
    
sisb_ship_reception_detail()


class stock_picking(osv.osv):
    _inherit = 'stock.picking'
    
    _columns = {
        'sisb_ship_id': fields.many2one('sisb.ship.reception', 'Ship Reception', ondelete='restrict', select=True),
        'sisb_detail_ship_id': fields.many2one('sisb.ship.reception.detail', 'Ship Reception Detail', ondelete='restrict', select=True),
    }
    
    _defaults = {
        'sisb_ship_id': False,
        'sisb_detail_ship_id': False,
    }
    
stock_picking()

class sisb_log_ship_reception(osv.osv):
    _name = "sisb.log.ship.reception"
    _description = "Log for ship reception"
    
    _columns = {
        'name': fields.char('Name', size=64, select=True),
        'ship_reception_id': fields.many2one('sisb.ship.reception', 'Ship Reception', select=True, ondelete='cascade'),
        'transition': fields.char('Transition', size=64, select=True),
        'description': fields.text('Description'),
        'responsible_uid': fields.many2one('res.users', 'User responsible of action', select=True),
        'is_notified': fields.boolean('Is notified', select=True),
    }
    
sisb_log_ship_reception()


class account_invoice(osv.osv):
    _inherit = "account.invoice"
    
    _columns = {
        'sisb_ship_reception_detail_ids': fields.many2many('sisb.ship.reception.detail', 'sisb_ship_reception_detail_invoice_rel', 'invoice_id', 'sisb_ship_reception_detail_id', 'Ship reception detail'),
    }
    
account_invoice()


class account_invoice_line(osv.osv):
    _inherit = "account.invoice.line"
    
    
    def move_line_get(self, cr, uid, invoice_id, context=None):
        
        #Tomar el costo desde el albaran asociado
        def sisb_get_price_cost_ship(cr, uid, inv, company_currency, i_line, product_cost_from_pick):
            cur_obj = self.pool.get('res.currency')
            
            if inv.currency_id.id != company_currency:
                price = cur_obj.compute(cr, uid, company_currency, inv.currency_id.id, product_cost_from_pick * i_line.quantity, context={'date': inv.date_invoice})
            else:
                price = product_cost_from_pick * i_line.quantity
            return price
        
        res = super(account_invoice_line,self).move_line_get(cr, uid, invoice_id, context=context)
        
        
        account_out_id = False
        #buscar si la factura pertenece a un detalle de buque
        inv = self.pool.get('account.invoice').browse(cr, uid, invoice_id, context=context)
        if inv.sisb_ship_reception_detail_ids:
            #sustituir la cuenta para colocar la contrapartida del albaran asociado (valuation_out_account_id de la ubicacion destino)
            detail_ship = inv.sisb_ship_reception_detail_ids and inv.sisb_ship_reception_detail_ids[0] or False
            if detail_ship:
                picking_out_brw = detail_ship.picking_out_ids
                if picking_out_brw:
                    #Garantizar que se tome el primer albaran generado
                    picking_out_brw = sorted(picking_out_brw, key=lambda picking_out: picking_out.id)
                    detail_pick_id = picking_out_brw[0]
                    detail_move_id = detail_pick_id.move_lines and detail_pick_id.move_lines[0] or False
                    if detail_move_id:
                        detail_location_dest_id = detail_move_id.location_dest_id and detail_move_id.location_dest_id or False
                        if detail_location_dest_id:
                            account_out_id = detail_location_dest_id.valuation_out_account_id and detail_location_dest_id.valuation_out_account_id.id or False
                            
                            if detail_move_id.price_unit:
                                #Costo al que se proceso el albaran
                                #Enviar la primera linea de la factura
                                company_currency = inv.company_id.currency_id.id
                                inv_line = sorted(inv.invoice_line, key=lambda inv_line_obj: inv_line_obj.id)
                                inv_line = inv_line[0]
                                price_unit_product = detail_move_id.price_unit
                                price_product = sisb_get_price_cost_ship(cr, uid, inv, company_currency, inv_line, detail_move_id.price_unit)
                                
                                #Cuenta contable de costo
                                # in both cases the credit account cacc will be the expense account
                                # first check the product, if empty check the category                                    
                                cacc = detail_move_id.product_id.product_tmpl_id.property_account_expense and detail_move_id.product_id.product_tmpl_id.property_account_expense.id
                                if not cacc:
                                    cacc = detail_move_id.product_id.categ_id.property_account_expense_categ and detail_move_id.product_id.categ_id.property_account_expense_categ.id
                                
                                if not cacc:
                                    cacc = self.get_product_category_account(cr,uid,detail_move_id.product_id.categ_id.id,"property_account_expense_categ",context=context)
                                        
                                if not cacc:
                                    raise osv.except_osv(_('Error !'),
                                            _('There is no account expense defined ' \
                                                    'for this product: "%s" (id:%d)') % \
                                                    (detail_move_id.product_id.name, detail_move_id.product_id.id,))
                                                    
                                ####
                                
                            else:
                                raise osv.except_osv(_('Error !'), _('There are inconsistencies in the price in stock picking.'))
                                
            #Buscar la cuenta a sustituir, que es la registrada en la cuenta variante de la ubicacion de salida de la sucursal
            account_x_desp = False
            if inv.journal_id.sale_shop_id:
                account_x_desp = inv.journal_id.sale_shop_id.warehouse_id.lot_output_id.account_location_variant_id and inv.journal_id.sale_shop_id.warehouse_id.lot_output_id.account_location_variant_id.id or False
            
            if account_x_desp and account_out_id:
                #Buscar la linea a reemplazar
                reemplazado01 = False
                reemplazado02 = False
                for x in res:
                    if x.get('account_id',False):
                        if x['account_id'] == account_x_desp:
                            x['account_id'] = account_out_id
                            x['price'] = price_product
                            x['price_unit'] = price_unit_product
                            reemplazado01 = True
                        elif x['account_id'] == cacc:
                            x['price'] = -1 * price_product
                            x['price_unit'] = price_unit_product
                            reemplazado02 = True
                if not reemplazado01 or not reemplazado02:
                    raise osv.except_osv(_('Error !'), _('There are inconsistencies in the accounts.'))
            else:
                raise osv.except_osv(_('Error !'), _('There is no valuation out account in destination location associated or there is no shop configurated in the journal.'))

        return res
    
account_invoice_line()
